#include <iostream>
#include <fstream>
//#include <cstdio>
//#include <cstdlib>
#include <vector>
#include <cmath>
#include "diff.h"
using namespace std;

Solver::Solver()
{
	NT    = 150;
	NX    = 50;
	NY    = 50;
	T_MAX = 1.0;
	X_MAX = 1.0;
	Y_MAX = 1.0;
	DT    = T_MAX/(double)NT;
	DX    = X_MAX/(double)NX;
	DY    = Y_MAX/(double)NY;

	T     = 1.0;
	PSI   = 0.0;
	PI    = 4.0*atan(1.0);

	// ړ_̐ (X+1)*(Y+1)
	node.resize(NX+1);
	for (int i=0; i<NX+1; i++) {
		node[i].resize(NY+1);
		for (int j=0; j<NY+1; j++) {
			node[i][j].index = i+j*NX;
			node[i][j].x = DX*(double)i;
			node[i][j].y = DY*(double)j;
			node[i][j].v.resize(NT+1);
		}
	}

	// Op`̐ (X)*(Y)*2
	element.resize(2*NX*NY);
	for (int i=0; i<NX; i++) {
		for (int j=0; j<NY; j++) {
			element[2*i+2*NY*j].node[0] = &node[i][j];
			element[2*i+2*NY*j].node[1] = &node[i+1][j+1];
			element[2*i+2*NY*j].node[2] = &node[i][j+1];
			element[2*i+2*NY*j+1].node[0] = &node[i][j];
			element[2*i+2*NY*j+1].node[1] = &node[i+1][j];
			element[2*i+2*NY*j+1].node[2] = &node[i+1][j+1];
		}
	}
}

Solver::~Solver()
{
}

void Solver::calc()
{
	int i, j, t;
	const double CX = (DT*DT)/(DX*DX);
	const double CY = (DT*DT)/(DY*DY);

	/* 0Ŗ߂ */
	for (i=0; i<=NX; i++) {
		for (j=0; j<=NY; j++) {
			for (t=0; t<=NT; t++) {
				node[i][j].v[t] = 0;
			}
		}
	}

	/* t=0 */
	for (i=0; i<=NX; i++) {
		for (j=0; j<=NY; j++) {
			if ((i == NX/2) && (j == NY/2)) {
				node[i][j].v[0] = 0.3;
			}
			//node[i][j].v[0] =
				//0.6*sin(PI*(double)i*DX) * sin(PI*(double)j*DY);
			//node[i][j].v[0] = 0.5*exp(-20.0*(i-NX/2)*(i-NX/2)*DX*DX)
			//	*exp(-20.0*(j-NY/2)*(j-NY/2)*DY*DY);
		}
	}

	/* t=1 */
	for (i=1; i<=NX-1; i++) {
		for (j=1; j<=NY-1; j++) {
			node[i][j].v[1] = node[i][j].v[0] + PSI*DT
				+ 0.5*T*CX*(node[i+1][j].v[0] - 2*node[i][j].v[0] + node[i-1][j].v[0])
				+ 0.5*T*CY*(node[i][j+1].v[0] - 2*node[i][j].v[0] + node[i][j-1].v[0]);
		}
	}
	
	for (t=1; t<=NT; t++) {
		for (i=1; i<=NX-1; i++) {
			for (j=1; j<=NY-1; j++) {
				node[i][j].v[t+1] = 2*node[i][j].v[t] - node[i][j].v[t-1]
					+ T*CX*(node[i+1][j].v[t] - 2*node[i][j].v[t] + node[i-1][j].v[t])
					+ T*CY*(node[i][j+1].v[t] - 2*node[i][j].v[t] + node[i][j-1].v[t]);
			}
		}
		/*
		// R[ۂ
		for (int i=1; i<=NX-1; i++) {
			node[i][0].v[t+1] = node[i][1].v[t+1];
			node[i][NY].v[t+1] = node[i][NY-1].v[t+1];
		}
		for (int i=1; i<=NY-1; i++) {
			node[0][i].v[t+1] = node[1][i].v[t+1];
			node[NX][i].v[t+1] = node[NY-1][i].v[t+1];
		}
		node[0][0].v[t+1]   = (node[1][0].v[t+1]+node[0][1].v[t+1])/2;
		node[NX][0].v[t+1]  = (node[NX-1][0].v[t+1]+node[NX][1].v[t+1])/2;
		node[0][NY].v[t+1]  = (node[1][NY].v[t+1]+node[0][NY-1].v[t+1])/2;
		node[NX][NY].v[t+1] = (node[NX-1][NY].v[t+1]+node[NX][NY-1].v[t+1])/2;
		*/
	}

}

void Solver::print()
{
	ofstream fs("result", ios::trunc);
	
	for (int i=0; i<=NX; i++) {
		for (int j=0; j<=NY; j++) {
			fs << i << '\t' << j;
			for (int t=0; t<=NT; t++) {
				fs << '\t' << node[i][j].v[t];
			}
			fs << endl;
		}
		fs << endl;
	}
	fs.close();
}

/*
int main()
{
	double u[NX][NY][NT];

	printf(".\n");
	
	init(u);
	calc(u);
	print(u);

	return 0;
}
*/
